<?php

namespace Drupal\currency_pbank;

use Drupal\Component\Serialization\Json;

/**
 * Privatbank rate trait.
 */
trait PbankRateTrait {

  /**
   * Privatbank exchange rates data.
   *
   * @var array
   */
  protected $pbankRates;

  /**
   * Gets currency codes privatbank provides rating for.
   *
   * @return string
   */
  public function getAvailableCurrencies() {
    return $this->pbankRates
      ? array_keys($this->pbankRates[array_key_first($this->pbankRates)])
      : array_keys($this->getPbankRates());
  }

  /**
   * Gets Privatbank currency exchange rate.
   *
   * @param string $currency_code
   *   Currency code.
   * @param string $type
   *   Rate type: buy or sale (buy by default).
   * @param bool $cashless
   *   True to get cashless exchange rate (default), false to get cash rate.
   *
   * @return float|null
   *   Exchange rate if found or NULL.
   *
   * @throws \InvalidArgumentException
   */
  public function getRate($currency_code, $type = 'buy', bool $cashless = TRUE) {
    if (!in_array($type, ['buy', 'sale'])) {
      throw new \InvalidArgumentException('Invalid Privatbank rate type.');
    }
    $rates = $this->getPbankRates($cashless);
    return isset($rates[$currency_code][$type]) ? (float) $rates[$currency_code][$type]: NULL;
  }

  /**
   * Gets Privatbank rates as it was received from Privatbank API.
   *
   * @param bool $cashless
   *   True to get cashless exchange rate (default), false to get cash rate.
   *
   * @return array
   *   Response from PrivatBank API keyed by currenty ID.
   */
  protected function getPbankRates(bool $cashless = TRUE) {
    $coursid = $cashless
      ? PrivatBankRateApiInterface::P24_API_RATE_TYPE_CASHLESS
      : PrivatBankRateApiInterface::P24_API_RATE_TYPE_CASH;

    if (!isset($this->pbankRates[$coursid])) {
      $this->pbankRates[$coursid] = [];
      $options = ['query' => ['exchange' => '', 'json' => '']];
      $options['query']['coursid'] = $coursid;
      $response = \Drupal::httpClient()->get(PrivatBankRateApiInterface::P24_API_PUBINFO_URL, $options);
      if (200 == $response->getStatusCode()) {
        $contents = Json::decode($response->getBody()->getContents());
        $this->pbankRates[$coursid] = array_combine(array_column($contents, 'ccy'), $contents);
      }
    }

    return $this->pbankRates[$coursid];
  }

}
