<?php

namespace Drupal\currency_pbank\Plugin\Currency\ExchangeRateProvider;

use Commercie\CurrencyExchange\FixedExchangeRateProviderTrait;
use Drupal\Component\Plugin\PluginBase;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\currency\Plugin\Currency\ExchangeRateProvider\ExchangeRateProviderInterface;
use Drupal\currency_pbank\PbankRateTrait;
use Drupal\currency_pbank\PrivatBankRateApiInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Provides manually entered exchange rates.
 *
 * @CurrencyExchangeRateProvider(
 *   id = "currency_pbank_rates",
 *   label = @Translation("Privatbank exchange rates"),
 *   operations_provider = "\Drupal\currency_pbank\Plugin\Currency\ExchangeRateProvider\PbankRatesOperationsProvider"
 * )
 */
class PbankRates extends PluginBase implements ExchangeRateProviderInterface, ContainerFactoryPluginInterface, PrivatBankRateApiInterface {

  use FixedExchangeRateProviderTrait;
  use PbankRateTrait;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new class instance.
   *
   * @param mixed[] $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed[] $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(array $configuration, $plugin_id, array $plugin_definition, ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function loadAll() {
    $pbank_rates = $this->configFactory->get('currency_pbank.exchanger')->get('pbank_rates');
    $rates = [];
    if ($pbank_rates) {
      foreach ($pbank_rates as $currency_code => $rate_type) {
        $cashless = $rate_type == static::P24_API_RATE_TYPE_CASHLESS;
        $rates[$currency_code]['UAH'] = $this->getRate($currency_code, 'buy', $cashless);
        $rates['UAH'][$currency_code] = bcdiv(1, $this->getRate($currency_code, 'sale', $cashless), 6);
      }
    }
    return $rates;
  }

  /**
   * Saves an exchange rate.
   *
   * @param string $currency_code
   *   Currency code.
   * @param string $rate_type
   *   Rate type ('buy' or 'sale').
   *
   * @return $this
   */
  public function save($currency_code, $rate_type) {
    $config = $this->configFactory->getEditable('currency_pbank.exchanger');
    $rates_data = $config->get('pbank_rates');
    $rates_data[$currency_code] = $rate_type;
    $config->set('pbank_rates', $rates_data);
    $config->save();

    return $this;
  }

  /**
   * Deletes an exchange rate.
   *
   * @param string $currency_code
   *   Currency code.
   */
  public function delete($currency_code) {
    $config = $this->configFactory->getEditable('currency_pbank.exchanger');
    $rates = $config->get('pbank_rates');
    unset($rates[$currency_code]);
    $config->set('pbank_rates', $rates);
    $config->save();
  }

}
